/**
  * PM, 2011
  * Proiect
  * Stocare date accelerometru pe card SD  
  * Prica Mihai - 335CC
  */
#include <avr/io.h>
#include <avr/interrupt.h>

#define F_CPU 1000000

#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include "diskio.h"
#include "pff.h"

/* configurare SPI */
#define SPI_PORT	PORTB
#define SPI_DDR		DDRB
#define DD_MOSI		5
#define DD_SCK		7
#define DD_SS		4

/* configurare card SD */
#define SD_CS_PORT   PORTB
#define SD_CS_DDR    DDRB
#define SD_CS_PIN    4

/* canalele ADC */
#define XCHANNEL 4
#define YCHANNEL 3
#define ZCHANNEL 2

/* in cazul in care se conecteaza un LCD */
//#define LCD		 1
#ifdef LCD
	#include "lcd.h"
#endif

/* dimensiunea unei linii din fisierul de output */
#define MAX_BUF		30

/* numarul de secunde intre masuratori */
#define NR_SECS		4

/* pentru a schimba starea led-ului de pe placuta */
#define change_led()				\
	PORTD = PORTD ^ _BV(PD7)		\

/* pentru detaliile sistemului de fisiere */
FATFS Fs;						/* sistemul de fisiere */
DIR Dir;						/* directorul */
FILINFO Fno;					/* fisierul */
WORD rb;			

/* valorile prin ADC */
int x_val, y_val, z_val;

/* buffer folosit la scrierea rezultatului pe card */
BYTE buffer[MAX_BUF];
int buf_index;
BOOL open = FALSE;

/* initializez ADC-ul pentru a citi valorile de la accelerometru */
void ADC_init(void)
{	
	/* referinta AVCC cu condesator extern pe pinul AREF */
	ADMUX &=~(1<<REFS1);
	ADMUX |= (1<<REFS0);
	
	ADCSRA |=   (1<<ADEN);

	/* prescaler de ceas la maxim */
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);	
}

/* initializez led-ul de pe placuta de baza */
void initLed()
{	
	DDRD  |= _BV(PD7);
	PORTD &= ~(_BV(PD7));
}

/* initializez timer-ul pentru intervalele de timp */
void initTimer()
{
	/* mode: CTC prescaler: 1024 */
	/* configurati timer-ul pentru NR_SECS secunde */
	/* activati intreruperea */
	OCR1A  = NR_SECS*977;
	TIMSK  = 0x10;
	TCCR1A = 0x00; 
	TCCR1B = 0x0D;
}

/* iau o valoare prin ADC de la unul din canale */
int ADC_get(uint8_t channel)
{	
	/* setez canalul de pe care vreau sa fac citirea */
	ADMUX &= ~((1<<MUX4)|(1<<MUX3)|(1<<MUX2)|(1<<MUX1)|(1<<MUX0));	
	ADMUX |= channel;

	ADCSRA |= (1<<ADSC);
	/* asteptati sa se termine o conversie */
	loop_until_bit_is_set(ADCSRA, ADIF);
		
	return ADC;
}

/* scriu o valoare in buffer */
void write_value(int val)
{
	buffer[buf_index--] = ';';
	do{
		buffer[buf_index--] = val%10 + '0';
		val/=10;
	}while(val);
}

/* transform cele trei valori primite prin ADC intr-un sir CSV */
void transform()
{	
	buf_index = MAX_BUF-1;	
	buffer[buf_index--] = 0;
	buffer[buf_index--] = '\n';
	write_value(z_val);
	write_value(y_val);
	write_value(x_val);
	
	while(buf_index>=0)
		buffer[buf_index--] = ' ';	
}

/* Rutina de tratare a intreruperii pentru timer */
ISR(TIMER1_COMPA_vect)
{
	/* schimb starea ledului de pe placuta de baza */
	change_led();

	/* citesc valorile de la ADC */
	x_val = ADC_get(XCHANNEL);
	y_val = ADC_get(YCHANNEL);	
	z_val = ADC_get(ZCHANNEL);	

	/* generez sirul pe care il voi scrie */
	transform();

	/* scriu bufferul in fisier */
	if(open==TRUE)
	{
#ifdef LCD
		LCD_writeInstruction(LCD_INSTR_clearDisplay);
		LCD_print("Am scris!");
#endif
		//pf_write (buffer, MAX_BUF, &rb);	
	}
}

/* Functia Main */
int main (void)
{	
	SPI_DDR  |=  _BV(DD_MOSI)|_BV(6);
	SPI_PORT &= ~_BV(DD_MOSI)|_BV(6);	

	open = FALSE;
	sei();							/* activez intreruperile */
	initLed();						/* initializez led-ul	*/
	ADC_init();						/* initializez ADC-ul	*/
	initTimer();					/* initializez timer-ul */
	
#ifdef LCD
	LCD_init();
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
#endif
	
	for(;;){
		if (pf_mount(&Fs)){
#ifdef LCD
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print("Failed mount!");
#endif
		}

		/* Deschid fisierul de baza */
		if (pf_opendir(&Dir, "")) {
#ifdef LCD
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print("Failed opendir!");
#endif
		}
		/* citesc toate fisierele din director */
		while(pf_readdir(&Dir, &Fno) && Fno.fname[0]) {	
			/* verific daca este fisierul meu */
			if (!(Fno.fattrib & (AM_DIR|AM_HID)) && strstr(Fno.fname, ".CSV")) {
#ifdef LCD
				LCD_writeInstruction(LCD_INSTR_clearDisplay);
				LCD_print("Failed open!");
#endif
				/* deschid fisierul */
				pf_open(Fno.fname);	
				open = TRUE;

				/* astept intreruperile sa scriu datele */
				while(1);
			}
		}
	}
	return 0;
}
